
#include "lop.h"
#include "adt.h"
#include "integral.h"
#include <math.h>
#include <stdio.h>

vertex v_st(mesh* _msh, hedge* _fc)
{
      return _msh->avertex[_fc->ipoint].ver;
}

vertex v_nd(mesh* _msh, hedge* _fc)
{
      return _msh->avertex[_msh->aedge[_fc->inext].ipoint].ver;
}

vertex v_rd(mesh* _msh, hedge* _fc)
{
      hedge* p;
      p = &_msh->aedge[_fc->inext];
      p = &_msh->aedge[p->inext];
      return _msh->avertex[p->ipoint].ver;
}

double area_triangle(vertex* _v1, vertex* _v2, vertex* _v3)
{
      return 0.5 * Sarlus(_v1,_v2,_v3);
}

double Integral(mesh* _msh, IntegralFunc _func, EvalFunc _evf)
{
      int i;
      double res = 0.0;
      for(i=0; i<_msh->nface; i++)
        res += _func(_msh, &_msh->aface[i], _evf);
      return res;
}

double F1(mesh* _msh, face* _fc, EvalFunc _evf)
{
      vertex v1;
      vertex v2;
      vertex v3;
      vertex  tmp;
      double mis;
      v1 = v_st(_msh, &_msh->aedge[_fc->iedge]);
      v2 = v_nd(_msh, &_msh->aedge[_fc->iedge]);
      v3 = v_rd(_msh, &_msh->aedge[_fc->iedge]);
      v1.z=1.0;
      v2.z=1.0;
      v3.z=1.0;

      tmp = 1/3.0 * (v1 + v2 + v3);

      mis = fabs(area_triangle(&v1, &v2, &v3));

      return mis*_evf(tmp.x, tmp.y);
}

double F2(mesh* _msh, face* _fc, EvalFunc _evf)
{
      vertex v1;
      vertex v2;
      vertex v3;
      vertex tmp1, tmp2, tmp3;
      double mis;
      v1 = v_st(_msh, &_msh->aedge[_fc->iedge]);
      v2 = v_nd(_msh, &_msh->aedge[_fc->iedge]);
      v3 = v_rd(_msh, &_msh->aedge[_fc->iedge]);
      v1.z=1.0;
      v2.z=1.0;
      v3.z=1.0;

      tmp1 = 1/2.0 * (v1 + v2);
      tmp2 = 1/2.0 * (v1 + v3);
      tmp3 = 1/2.0 * (v3 + v2);

      mis = fabs(area_triangle(&v1, &v2, &v3));

      return mis*1/3.0*(_evf(tmp1.x, tmp1.y) + _evf(tmp2.x, tmp2.y)+ _evf(tmp3.x, tmp3.y));
}

double F3(mesh* _msh, face* _fc, EvalFunc _evf)
{
      vertex v1;
      vertex v2;
      vertex v3;
      vertex tmp1, tmp2, tmp3;
      double mis;
      v1 = v_st(_msh, &_msh->aedge[_fc->iedge]);
      v2 = v_nd(_msh, &_msh->aedge[_fc->iedge]);
      v3 = v_rd(_msh, &_msh->aedge[_fc->iedge]);
      v1.z=1.0;
      v2.z=1.0;
      v3.z=1.0;

      tmp1 = 2.0/3 * v1 + 1.0/6 * (v3 + v2);
      tmp2 = 2.0/3 * v2 + 1.0/6 * (v3 + v1);
      tmp3 = 2.0/3 * v3 + 1.0/6 * (v1 + v2);

      mis = fabs(area_triangle(&v1, &v2, &v3));

      return mis*1/3.0*(_evf(tmp1.x, tmp1.y) + _evf(tmp2.x, tmp2.y)+ _evf(tmp3.x, tmp3.y));
}

double F4(mesh* _msh, face* _fc, EvalFunc _evf)
{
      vertex v1;
      vertex v2;
      vertex v3;
      vertex tmp1, tmp2, tmp3, tmp4;
      double mis;
      v1 = v_st(_msh, &_msh->aedge[_fc->iedge]);
      v2 = v_nd(_msh, &_msh->aedge[_fc->iedge]);
      v3 = v_rd(_msh, &_msh->aedge[_fc->iedge]);
      v1.z=1.0;
      v2.z=1.0;
      v3.z=1.0;

      tmp1 = 3.0/5 * v1 + 1.0/5 * (v3 + v2);
      tmp2 = 3.0/5 * v2 + 1.0/5 * (v3 + v1);
      tmp3 = 3.0/5 * v3 + 1.0/5 * (v1 + v2);
      tmp4 = 1/3.0 * (v1 + v2 + v3);

      mis = fabs(area_triangle(&v1, &v2, &v3));

      return mis*(25/48.0*(_evf(tmp1.x, tmp1.y) + _evf(tmp2.x, tmp2.y)+ _evf(tmp3.x, tmp3.y)) - 9.0/16 * _evf(tmp4.x, tmp4.y));
}

